@@ -9,37 +9,76 @@ mod config_test;
 extern crate serde_json;
 
 use serde::Deserialize;
-use std::collections::HashMap;
 use std::io::{BufReader, Read};
 use std::sync::Arc;
 use vrp_core::models::common::SingleDimLoad;
 use vrp_core::models::Problem;
 use vrp_core::solver::mutation::*;
-use vrp_core::solver::selection::NaiveSelection;
+use vrp_core::solver::population::*;
 use vrp_core::solver::{Builder, Telemetry, TelemetryMode};
-use vrp_core::utils::get_cpus;
+use vrp_core::utils::{get_cpus, DefaultRandom};
 
 /// An algorithm configuration.
 #[derive(Clone, Deserialize, Debug)]
 pub struct Config {
-    /// Specifies population configuration.
-    pub population: Option<PopulationConfig>,
-    /// Specifies mutation operator configuration.
-    pub selection: Option<SelectionConfig>,
-    /// Specifies mutation operator configuration.
-    pub mutation: Option<MutationConfig>,
+    /// Specifies evolution configuration.
+    pub evolution: Option<EvolutionConfig>,
+    /// Specifies mutation operator type.
+    pub mutation: Option<MutationType>,
     /// Specifies algorithm termination configuration.
     pub termination: Option<TerminationConfig>,
     /// Specifies telemetry configuration.
     pub telemetry: Option<TelemetryConfig>,
 }
 
-/// A population configuration.
+/// An evolution configuration.
 #[derive(Clone, Deserialize, Debug)]
 #[serde(rename_all = "camelCase")]
-pub struct PopulationConfig {
+pub struct EvolutionConfig {
     initial: Option<InitialConfig>,
-    max_size: Option<usize>,
+    population: Option<PopulationType>,
+}
+
+#[derive(Clone, Deserialize, Debug)]
+#[serde(tag = "type")]
+#[serde(rename_all = "camelCase")]
+pub enum PopulationType {
+    /// A basic population which sorts individuals based on their
+    /// dominance order.
+    #[serde(rename(deserialize = "elitism"))]
+    #[serde(rename_all = "camelCase")]
+    Elitism {
+        /// Max population size. Default is 4.
+        max_size: Option<usize>,
+        /// Selection size. Default is number of cpus.
+        selection_size: Option<usize>,
+    },
+
+    /// A population algorithm based on SOM.
+    #[serde(rename(deserialize = "rosomaxa"))]
+    #[serde(rename_all = "camelCase")]
+    Rosomaxa {
+        /// Selection size. Default is number of cpus.
+        selection_size: Option<usize>,
+        /// Elite population size. Default is 2.
+        max_elite_size: Option<usize>,
+        /// Node population size. Default is 2.
+        max_node_size: Option<usize>,
+        /// Spread factor. Default is 0.25.
+        spread_factor: Option<f64>,
+        /// The reduction factor. Default is 0.1.
+        reduction_factor: Option<f64>,
+        /// Distribution factor. Default is 0.25.
+        distribution_factor: Option<f64>,
+        /// Learning rate. Default is 0.1.
+        learning_rate: Option<f64>,
+        /// A rebalance memory. Default is 500.
+        rebalance_memory: Option<usize>,
+        /// A rebalance count. Default is 10.
+        rebalance_count: Option<usize>,
+        /// An exploration phase ratio. Default is 0.9.
+        exploration_ratio: Option<f64>,
+    },
 }
 
 /// An initial solution configuration.
@@ -50,15 +89,6 @@ pub struct InitialConfig {
     pub methods: Option<Vec<RecreateMethod>>,
 }
 
-/// A selection configuration.
-#[derive(Clone, Deserialize, Debug)]
-pub struct SelectionConfig {
-    /// A name of used selection from the collection.
-    name: String,
-    /// Collection of available selection algorithms.
-    collection: Vec<SelectionType>,
-}
-
 /// A selection operator configuration.
 #[derive(Clone, Deserialize, Debug)]
 #[serde(tag = "type")]
@@ -66,22 +96,11 @@ pub struct SelectionConfig {
 pub enum SelectionType {
     #[serde(rename(deserialize = "naive"))]
     Naive {
-        /// A name of selection operator.
-        name: String,
         /// A size of offspring.
         offspring_size: Option<usize>,
     },
 }
 
-/// A mutation configuration.
-#[derive(Clone, Deserialize, Debug)]
-pub struct MutationConfig {
-    /// A name of used mutation from the collection.
-    name: String,
-    /// Collection of available mutation metaheuristics.
-    collection: Vec<MutationType>,
-}
-
 /// A mutation operator configuration.
 #[derive(Clone, Deserialize, Debug)]
 #[serde(tag = "type")]
@@ -90,23 +109,32 @@ pub enum MutationType {
     /// probability weights.
     #[serde(rename(deserialize = "composite"))]
     Composite {
-        /// A name of metaheurisic instance.
-        name: String,
+        /// Probability.
+        probability: f64,
         /// A collection of inner metaheuristics.
-        inners: Vec<NameWeight>,
+        inners: Vec<MutationType>,
+    },
+
+    /// A local search heuristic.
+    #[serde(rename(deserialize = "local-search"))]
+    LocalSearch {
+        /// Probability of the group.
+        probability: f64,
+        /// Amount of times one of operators is applied.
+        times: MinMaxConfig,
+        /// Local search operator.
+        operators: Vec<LocalOperatorType>,
     },
 
     /// A ruin and recreate metaheuristic settings.
     #[serde(rename(deserialize = "ruin-recreate"))]
     RuinRecreate {
-        /// A name of metaheurisic instance.
-        name: String,
+        /// Probability.
+        probability: f64,
         /// Ruin methods.
         ruins: Vec<RuinGroupConfig>,
         /// Recreate methods.
         recreates: Vec<RecreateMethod>,
-        /// Local search config.
-        locals: LocalSearchConfig,
     },
 }
 
@@ -162,36 +190,19 @@ pub enum RecreateMethod {
     /// Nearest neighbour method.
     #[serde(rename(deserialize = "nearest"))]
     Nearest { weight: usize },
+    /// Farthest insertion method.
+    #[serde(rename(deserialize = "farthest"))]
+    Farthest { weight: usize },
     #[serde(rename(deserialize = "perturbation"))]
     Perturbation { weight: usize, probability: f64, min: f64, max: f64 },
     #[serde(rename(deserialize = "regret"))]
     Regret { weight: usize, start: usize, end: usize },
 }
 
-/// A local search configuration
-#[derive(Clone, Deserialize, Debug)]
-pub struct LocalSearchConfig {
-    /// Pre ruin local search.
-    pre_ruin: LocalSearchGroupConfig,
-    /// Post recreate local search.
-    post_recreate: LocalSearchGroupConfig,
-}
-
-/// A local search group configuration
-#[derive(Clone, Deserialize, Debug)]
-pub struct LocalSearchGroupConfig {
-    /// Probability of the group.
-    probability: f64,
-    /// Amount of times one of operators is applied.
-    times: MinMaxConfig,
-    /// List of operators.
-    operators: Vec<LocalSearchOperator>,
-}
-
 /// A local search configuration.
 #[derive(Clone, Deserialize, Debug)]
 #[serde(tag = "type")]
-pub enum LocalSearchOperator {
+pub enum LocalOperatorType {
     #[serde(rename(deserialize = "inter-route-best"))]
     InterRouteBest { weight: usize, noise: NoiseConfig },
 
@@ -200,6 +211,9 @@ pub enum LocalSearchOperator {
 
     #[serde(rename(deserialize = "intra-route-random"))]
     IntraRouteRandom { weight: usize, noise: NoiseConfig },
+
+    #[serde(rename(deserialize = "push-route-departure"))]
+    PushRouteDeparture { weight: usize, offset: f64 },
 }
 
 #[derive(Clone, Deserialize, Debug)]
@@ -212,9 +226,9 @@ pub struct NoiseConfig {
 #[derive(Clone, Deserialize, Debug)]
 #[serde(rename_all = "camelCase")]
 pub struct TerminationConfig {
-    max_time: Option<usize>,
-    max_generations: Option<usize>,
-    variation: Option<VariationConfig>,
+    pub max_time: Option<usize>,
+    pub max_generations: Option<usize>,
+    pub variation: Option<VariationConfig>,
 }
 
 #[derive(Clone, Deserialize, Debug)]
@@ -238,6 +252,8 @@ pub struct LoggingConfig {
     log_best: Option<usize>,
     /// Specifies how often population is logged. Default is 1000 (generations).
     log_population: Option<usize>,
+    /// Specifies whether population should be dumped.
+    dump_population: Option<bool>,
 }
 
 #[derive(Clone, Deserialize, Debug)]
@@ -263,13 +279,14 @@ pub struct NameWeight {
 
 impl Default for Config {
     fn default() -> Self {
-        Self { population: None, selection: None, mutation: None, termination: None, telemetry: None }
+        Self { evolution: None, mutation: None, termination: None, telemetry: None }
     }
 }
 
-fn configure_from_population(
+fn configure_from_evolution(
     mut builder: Builder,
-    population_config: &Option<PopulationConfig>,
+    population_config: &Option<EvolutionConfig>,
+    problem: Arc<Problem>,
 ) -> Result<Builder, String> {
     if let Some(config) = population_config {
         if let Some(initial) = &config.initial {
@@ -282,87 +299,76 @@ fn configure_from_population(
             );
         }
 
-        if let Some(population_size) = &config.max_size {
-            builder = builder.with_population_size(*population_size);
+        if let Some(variation) = &config.population {
+            // TODO pass random from outside
+            let random = Arc::new(DefaultRandom::default());
+
+            let population = match &variation {
+                PopulationType::Elitism { max_size, selection_size } => Box::new(Elitism::new(
+                    problem,
+                    random,
+                    max_size.unwrap_or(4),
+                    selection_size.unwrap_or_else(get_cpus),
+                )),
+                PopulationType::Rosomaxa {
+                    max_elite_size,
+                    max_node_size,
+                    spread_factor,
+                    reduction_factor,
+                    distribution_factor,
+                    learning_rate,
+                    selection_size,
+                    rebalance_memory,
+                    rebalance_count,
+                    exploration_ratio,
+                } => {
+                    let mut config = RosomaxaConfig::default();
+                    if let Some(max_elite_size) = max_elite_size {
+                        config.elite_size = *max_elite_size;
+                    }
+                    if let Some(max_node_size) = max_node_size {
+                        config.node_size = *max_node_size;
+                    }
+                    if let Some(spread_factor) = spread_factor {
+                        config.spread_factor = *spread_factor;
+                    }
+                    if let Some(reduction_factor) = reduction_factor {
+                        config.reduction_factor = *reduction_factor;
+                    }
+                    if let Some(distribution_factor) = distribution_factor {
+                        config.distribution_factor = *distribution_factor;
+                    }
+                    if let Some(learning_rate) = learning_rate {
+                        config.learning_rate = *learning_rate;
+                    }
+                    if let Some(selection_size) = selection_size {
+                        config.selection_size = *selection_size;
+                    }
+                    if let Some(rebalance_memory) = rebalance_memory {
+                        config.rebalance_memory = *rebalance_memory;
+                    }
+                    if let Some(rebalance_count) = rebalance_count {
+                        config.rebalance_count = *rebalance_count;
+                    }
+                    if let Some(exploration_ratio) = exploration_ratio {
+                        config.exploration_ratio = *exploration_ratio;
+                    }
+
+                    Rosomaxa::new_with_fallback(problem, random, config)
+                }
+            };
+
+            builder = builder.with_population(population);
         }
     }
 
     Ok(builder)
 }
 
-fn configure_from_selection(
-    mut builder: Builder,
-    selection_config: &Option<SelectionConfig>,
-) -> Result<Builder, String> {
-    if let Some(config) = selection_config {
-        let selections = config
-            .collection
-            .iter()
-            .map(|item| match item {
-                SelectionType::Naive { name, offspring_size } => {
-                    (name, Arc::new(NaiveSelection::new(offspring_size.unwrap_or_else(get_cpus))))
-                }
-            })
-            .collect::<HashMap<_, _>>();
-
-        let selection =
-            selections.get(&config.name).cloned().ok_or_else(|| format!("cannot find {} selection", config.name))?;
-
-        builder = builder.with_selection(selection);
-    }
-
-    Ok(builder)
-}
-
-type NamedMutations = HashMap<String, Arc<dyn Mutation + Send + Sync>>;
-
-fn configure_from_mutation(mut builder: Builder, mutation_config: &Option<MutationConfig>) -> Result<Builder, String> {
+fn configure_from_mutation(mut builder: Builder, mutation_config: &Option<MutationType>) -> Result<Builder, String> {
     if let Some(config) = mutation_config {
-        let get_mutation_by_name = |mutations: &NamedMutations, name: &String| {
-            mutations
-                .get(name)
-                .cloned()
-                .ok_or_else(|| format!("cannot find {} mutation, make sure that it is defined before used", name))
-        };
-        let mutations = config.collection.iter().try_fold::<_, _, Result<_, String>>(
-            NamedMutations::default(),
-            |mut mutations, type_cfg| {
-                let (name, mutation): (_, Arc<dyn Mutation + Send + Sync>) = match type_cfg {
-                    MutationType::RuinRecreate { name, ruins, recreates, locals } => {
-                        let ruin = Box::new(CompositeRuin::new(
-                            ruins.iter().map(|g| create_ruin_group(&builder.config.problem, g)).collect(),
-                        ));
-                        let recreate = Box::new(CompositeRecreate::new(
-                            recreates.iter().map(|r| create_recreate_method(r)).collect(),
-                        ));
-                        (
-                            name.clone(),
-                            Arc::new(RuinAndRecreate::new(
-                                recreate,
-                                ruin,
-                                create_local_search(&locals.pre_ruin),
-                                create_local_search(&locals.post_recreate),
-                            )),
-                        )
-                    }
-                    MutationType::Composite { name, inners } => {
-                        let inners = inners
-                            .iter()
-                            .map(|nw| get_mutation_by_name(&mutations, &nw.name).map(|mutation| (mutation, nw.weight)))
-                            .collect::<Result<Vec<_>, _>>()?;
-                        (name.clone(), Arc::new(CompositeMutation::new(inners)))
-                    }
-                };
-
-                mutations.insert(name, mutation);
-
-                Ok(mutations)
-            },
-        )?;
-
-        let mutation = get_mutation_by_name(&mutations, &config.name)?.clone();
-
-        builder = builder.with_mutation(mutation);
+        let mutation = create_mutation(&builder.config.problem, config)?.0;
+        builder = builder.with_mutation(mutation)
     }
 
     Ok(builder)
@@ -384,6 +390,7 @@ fn configure_from_termination(
 fn create_recreate_method(method: &RecreateMethod) -> (Box<dyn Recreate + Send + Sync>, usize) {
     match method {
         RecreateMethod::Cheapest { weight } => (Box::new(RecreateWithCheapest::default()), *weight),
+        RecreateMethod::Farthest { weight } => (Box::new(RecreateWithFarthest::default()), *weight),
         RecreateMethod::SkipBest { weight, start, end } => (Box::new(RecreateWithSkipBest::new(*start, *end)), *weight),
         RecreateMethod::Blinks { weight } => (Box::new(RecreateWithBlinks::<SingleDimLoad>::default()), *weight),
         RecreateMethod::Gaps { weight, min } => (Box::new(RecreateWithGaps::new(*min)), *weight),
@@ -395,6 +402,30 @@ fn create_recreate_method(method: &RecreateMethod) -> (Box<dyn Recreate + Send +
     }
 }
 
+fn create_mutation(
+    problem: &Arc<Problem>,
+    mutation: &MutationType,
+) -> Result<(Arc<dyn Mutation + Send + Sync>, f64), String> {
+    Ok(match mutation {
+        MutationType::RuinRecreate { probability, ruins, recreates } => {
+            let ruin = Box::new(CompositeRuin::new(ruins.iter().map(|g| create_ruin_group(problem, g)).collect()));
+            let recreate =
+                Box::new(CompositeRecreate::new(recreates.iter().map(|r| create_recreate_method(r)).collect()));
+            (Arc::new(RuinAndRecreate::new(recreate, ruin)), *probability)
+        }
+        MutationType::LocalSearch { probability, times, operators: inners } => {
+            let operator = create_local_search(times, inners);
+            (Arc::new(LocalSearch::new(operator)), *probability)
+        }
+        MutationType::Composite { probability, inners } => {
+            let inners =
+                inners.iter().map(|mutation| create_mutation(problem, mutation)).collect::<Result<Vec<_>, _>>()?;
+
+            (Arc::new(CompositeMutation::new(vec![(inners, 1)])), *probability)
+        }
+    })
+}
+
 fn create_ruin_group(problem: &Arc<Problem>, group: &RuinGroupConfig) -> RuinGroup {
     (group.methods.iter().map(|r| create_ruin_method(problem, r)).collect(), group.weight)
 }
@@ -423,24 +454,26 @@ fn create_ruin_method(problem: &Arc<Problem>, method: &RuinMethod) -> (Arc<dyn R
     }
 }
 
-fn create_local_search(group: &LocalSearchGroupConfig) -> (Box<dyn LocalSearch + Send + Sync>, f64) {
-    let operators = group
-        .operators
+fn create_local_search(times: &MinMaxConfig, inners: &[LocalOperatorType]) -> Box<dyn LocalOperator + Send + Sync> {
+    let operators = inners
         .iter()
-        .map::<(Box<dyn LocalSearch + Send + Sync>, usize), _>(|op| match op {
-            LocalSearchOperator::InterRouteBest { weight, noise } => {
+        .map::<(Box<dyn LocalOperator + Send + Sync>, usize), _>(|op| match op {
+            LocalOperatorType::InterRouteBest { weight, noise } => {
                 (Box::new(ExchangeInterRouteBest::new(noise.probability, noise.min, noise.max)), *weight)
             }
-            LocalSearchOperator::InterRouteRandom { weight, noise } => {
+            LocalOperatorType::InterRouteRandom { weight, noise } => {
                 (Box::new(ExchangeInterRouteRandom::new(noise.probability, noise.min, noise.max)), *weight)
             }
-            LocalSearchOperator::IntraRouteRandom { weight, noise } => {
+            LocalOperatorType::IntraRouteRandom { weight, noise } => {
                 (Box::new(ExchangeIntraRouteRandom::new(noise.probability, noise.min, noise.max)), *weight)
             }
+            LocalOperatorType::PushRouteDeparture { weight, offset } => {
+                (Box::new(PushRouteDeparture::new(*offset)), *weight)
+            }
         })
         .collect::<Vec<_>>();
 
-    (Box::new(CompositeLocalSearch::new(operators, group.times.min, group.times.max)), group.probability)
+    Box::new(CompositeLocalOperator::new(operators, times.min, times.max))
 }
 
 fn configure_from_telemetry(builder: Builder, telemetry_config: &Option<TelemetryConfig>) -> Result<Builder, String> {
@@ -454,19 +487,22 @@ fn configure_from_telemetry(builder: Builder, telemetry_config: &Option<Telemetr
         track_population: track_population.unwrap_or(TRACK_POPULATION),
     };
 
-    let create_logging = |log_best: &Option<usize>, log_population: &Option<usize>| TelemetryMode::OnlyLogging {
-        logger: create_logger(),
-        log_best: log_best.unwrap_or(LOG_BEST),
-        log_population: log_population.unwrap_or(LOG_POPULATION),
+    let create_logging = |log_best: &Option<usize>, log_population: &Option<usize>, dump_population: &Option<bool>| {
+        TelemetryMode::OnlyLogging {
+            logger: create_logger(),
+            log_best: log_best.unwrap_or(LOG_BEST),
+            log_population: log_population.unwrap_or(LOG_POPULATION),
+            dump_population: dump_population.unwrap_or(false),
+        }
     };
 
     let telemetry_mode = match telemetry_config.as_ref().map(|t| (&t.logging, &t.metrics)) {
         Some((None, Some(MetricsConfig { enabled, track_population }))) if *enabled => create_metrics(track_population),
-        Some((Some(LoggingConfig { enabled, log_best, log_population }), None)) if *enabled => {
-            create_logging(log_best, log_population)
+        Some((Some(LoggingConfig { enabled, log_best, log_population, dump_population }), None)) if *enabled => {
+            create_logging(log_best, log_population, dump_population)
         }
         Some((
-            Some(LoggingConfig { enabled: logging_enabled, log_best, log_population }),
+            Some(LoggingConfig { enabled: logging_enabled, log_best, log_population, dump_population }),
             Some(MetricsConfig { enabled: metrics_enabled, track_population }),
         )) => match (logging_enabled, metrics_enabled) {
             (true, true) => TelemetryMode::All {
@@ -474,8 +510,9 @@ fn configure_from_telemetry(builder: Builder, telemetry_config: &Option<Telemetr
                 log_best: log_best.unwrap_or(LOG_BEST),
                 log_population: log_population.unwrap_or(LOG_POPULATION),
                 track_population: track_population.unwrap_or(TRACK_POPULATION),
+                dump_population: dump_population.unwrap_or(false),
             },
-            (true, false) => create_logging(log_best, log_population),
+            (true, false) => create_logging(log_best, log_population, dump_population),
             (false, true) => create_metrics(track_population),
             _ => TelemetryMode::None,
         },
@@ -500,11 +537,10 @@ pub fn create_builder_from_config_file<R: Read>(
 
 /// Creates a solver `Builder` from config.
 pub fn create_builder_from_config(problem: Arc<Problem>, config: &Config) -> Result<Builder, String> {
-    let mut builder = Builder::new(problem);
+    let mut builder = Builder::new(problem.clone());
 
     builder = configure_from_telemetry(builder, &config.telemetry)?;
-    builder = configure_from_population(builder, &config.population)?;
-    builder = configure_from_selection(builder, &config.selection)?;
+    builder = configure_from_evolution(builder, &config.evolution, problem)?;
     builder = configure_from_mutation(builder, &config.mutation)?;
     builder = configure_from_termination(builder, &config.termination)?;
 
